window.Gleam.initSearch([{"doc":"pevensie_postgres","title":"pevensie_postgres","content":"# Pevensie Postgres Driver\n\n[![Package Version](https://img.shields.io/hexpm/v/pevensie_postgres)](https://hex.pm/packages/pevensie_postgres)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/pevensie_postgres/)\n\nThe official PostgreSQL driver for Pevensie. It provides driver\nimplementations for Pevensie modules to be used with Postgres\ndatabases.\n\nCurrently provides drivers for:\n\n- [Pevensie Auth](https://hexdocs.pm/pevensie/pevensie/auth.html)\n- [Pevensie Cache](https://hexdocs.pm/pevensie/pevensie/cache.html)\n\n## Getting Started\n\nConfigure your driver to connect to your database using the\n[`PostgresConfig`](https://hexdocs.pm/pevensie_postgres/pevensie/postgres.html#PostgresConfig)\ntype. You can use the [`default_config`](https://hexdocs.pm/pevensie_postgres/pevensie/postgres.html#default_config)\nfunction to get a default configuration for connecting to a local\nPostgres database with sensible concurrency defaults.\n\n```gleam\nimport pevensie/postgres.{type PostgresConfig}\n\npub fn main() {\n  let config = PostgresConfig(\n    ..postgres.default_config(),\n    host: &quot;db.pevensie.dev&quot;,\n    database: &quot;my_database&quot;,\n  )\n  // ...\n}\n```\n\nCreate a new driver using one of the `new_&lt;driver&gt;_driver` functions\nprovided by this module. You can then use the driver with Pevensie\nmodules.\n\n```gleam\nimport pevensie/postgres.{type PostgresConfig}\nimport pevensie/auth.{type PevensieAuth}\n\npub fn main() {\n  let config = PostgresConfig(\n    ..postgres.default_config(),\n    host: &quot;db.pevensie.dev&quot;,\n    database: &quot;my_database&quot;,\n  )\n  let driver = postgres.new_auth_driver(config)\n  let pevensie_auth = auth.new(\n    driver:,\n    user_metadata_decoder:,\n    user_metadata_encoder:,\n    cookie_key: &quot;super secret signing key&quot;,\n  )\n  // ...\n}\n```\n\n## Connection Management\n\nWhen called with the Postgres driver, the [`connect`](https://hexdocs.pm/pevensie/pevensie/auth.html#connect) function\nprovided by Pevensie Auth will create a connection pool for the database. This can be called\nonce on boot, and will be reused for the lifetime of the application.\n\nThe [`disconnect`](https://hexdocs.pm/pevensie/pevensie/auth.html#disconnect) function will close the connection pool.\n\n## Tables\n\nThis driver creates tables in the `pevensie` schema to store user\ndata, sessions, and cache data.\n\nThe current tables created by this driver are:\n\n| Table Name | Description |\n| ---------- | ----------- |\n| `cache` | Stores cache data. This table is unlogged, so data will be lost when the database stops. |\n| `one_time_token` | Stores one time tokens. Uses soft deletions. |\n| `session` | Stores session data. |\n| `user` | Stores user data. Uses soft deletions. |\n| `module_version` | Stores the current versions of the tables required by this driver. Versions are stored as dates. |\n\n### Types\n\nGenerally, the Postgres driver makes a best effort to map the types\nused by Pevensie to best-practice Postgres types. Generally, columns\nare non-nullable unless the Gleam type is `Option(a)`. The following\ntypes are mapped to Postgres types:\n\n| Gleam Type | Postgres Type |\n| ---------- | ------------- |\n| Any resource ID | `UUID` (generated as UUIDv7) |\n| `String` | `text` |\n| `tempo.DateTime` (from [`gtempo`](https://hexdocs.pm/gtempo)) | `timestamptz` |\n| Record types (e.g. `user_metadata`) | `jsonb` |\n| `pevensie/net.IpAddr` | `inet` |\n\n## Migrations\n\nYou can run migrations against your\ndatabase using the provided CLI:\n\n```sh\ngleam run -m pevensie/postgres migrate --addr=&lt;connection_string&gt; auth cache\n```\n\nThe required SQL statements will be printed to the console for use with other migration\ntools like [`dbmate`](https://github.com/amacneil/dbmate). Alternatively, you can\napply the migrations directly using the `--apply` flag.\n\n## Implementation Details\n\nThis driver uses the [pog](https://github.com/lpil/pog) library for interacting\nwith Postgres.\n\nAll IDs are stored as UUIDs, and are generated using using a UUIDv7 implementation\nmade available by [Fabio Lima](https://github.com/fabiolimace) under the MIT license.\nThe implementation is available [here](https://gist.github.com/fabiolimace/515a0440e3e40efeb234e12644a6a346).\n\n### Pevensie Auth\n\nThe `user` table follows the structure of the [`User`](https://hexdocs.pm/pevensie/pevensie/user.html#User)\ntype. The `user_metadata` column is a JSONB column, and is used to store\nany custom user metadata.\n\nIt also contains a `deleted_at` column, which is used to mark users as deleted,\nrather than deleting the row from the database.\n\nAlongside the primary key, the `user` table has unique indexes on the\n`email` and `phone_number` columns. These are partial, and only index\nwhere values are provided. They also include the `deleted_at` column,\nso users can sign up with the same email if a user with that email\nhas been deleted.\n\nThe `session` table follows the structure of the [`Session`](https://hexdocs.pm/pevensie/pevensie/session.html#Session)\ntype. The `user_id` column is a foreign key referencing the `user` table. If\nan expired session is read, the `get_session` function will return `None`, and\ndelete the expired session from the database.\n\n#### Searching for users\n\nThe `list_users` function provided by Pevensie Auth allows you to search for users\nby ID, email or phone number. With the Postgres driver, the lists in you\n[`UserSearchFields`](https://hexdocs.pm/pevensie/pevensie/user.html#UserSearchFields) argument are processed\nusing a `like any()` where clause. This means that you can search for users\nby providing a list of values, and the driver will search for users with\nany of those values.\n\nYou can also use Postgres `like` wildcards in your search values. For example,\nif you search for users with an email ending in `@example.com`, you can use\n`%@example.com` as the search value.\n\n### Pevensie Cache\n\nThe `cache` table is an [`unlogged`](https://www.postgresql.org/docs/current/sql-createtable.html#SQL-CREATETABLE-UNLOGGED)\ntable. This ensures that writes are fast, but data is lost when the database\nis stopped. Do not use Pevensie Cache for data you need to keep indefinitely.\n\nThe table&#39;s primary key is a composite of the `resource_type` and `key`\ncolumns. This allows you to store multiple values for a given key, and\nretrieve them all at once.\n\nWrites to the cache will overwrite any existing value for the given\nresource type and key. This driver does not keep a version history.\n\nIf an expired value is read, the `get` function will return `None`, and\ndelete the expired value from the cache.\n\n### Custom Queries\n\nIf you wish to query the `user` or `session` tables directly, this driver\nprovides helper utilities for doing so.\n\nThe [`user_decoder`](https://hexdocs.pm/pevensie_postgres/pevensie/postgres.html#user_decoder) and\n[`session_decoder`](https://hexdocs.pm/pevensie_postgres/pevensie/postgres.html#session_decoder) functions\ncan be used to decode selections from the `user` and `session` tables, respectively.\n\nThe [`user_select_fields`](https://hexdocs.pm/pevensie_postgres/pevensie/postgres.html#user_select_fields)\nand [`session_select_fields`](https://hexdocs.pm/pevensie_postgres/pevensie/postgres.html#session_select_fields)\nvariables contain the SQL used to select fields from the `user` and `session`\ntables for use with the `user_decoder` and `session_decoder` functions.\n","url":"index.html"},{"doc":"pevensie/postgres","title":"IpVersion","content":"pub type IpVersion {\n  Ipv4\n  Ipv6\n}\n An IP version for a [`PostgresConfig`](#PostgresConfig).\nIpv4\n\n\nIpv6\n\n\nSynonyms:\npevensie/postgres.IpVersion\npevensie/postgres IpVersion","url":"pevensie/postgres.html#IpVersion"},{"doc":"pevensie/postgres","title":"Postgres","content":"pub opaque type Postgres\n The Postgres driver.\n\nSynonyms:\npevensie/postgres.Postgres\npevensie/postgres Postgres","url":"pevensie/postgres.html#Postgres"},{"doc":"pevensie/postgres","title":"PostgresConfig","content":"pub type PostgresConfig {\n  PostgresConfig(\n    host: String,\n    port: Int,\n    database: String,\n    user: String,\n    password: Option(String),\n    ssl: Bool,\n    connection_parameters: List(#(String, String)),\n    pool_size: Int,\n    queue_target: Int,\n    queue_interval: Int,\n    idle_interval: Int,\n    trace: Bool,\n    ip_version: IpVersion,\n    default_timeout: Int,\n  )\n}\n Configuration for connecting to a Postgres database.\n\n Use the [`default_config`](/pevensie/drivers/postgres.html#default_config)\n function to get a default configuration for connecting to a local\n Postgres database with sensible concurrency defaults.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   // ...\n }\n \nPostgresConfig(\n  host: String,\n  port: Int,\n  database: String,\n  user: String,\n  password: Option(String),\n  ssl: Bool,\n  connection_parameters: List(#(String, String)),\n  pool_size: Int,\n  queue_target: Int,\n  queue_interval: Int,\n  idle_interval: Int,\n  trace: Bool,\n  ip_version: IpVersion,\n  default_timeout: Int,\n)\n\n\nSynonyms:\npevensie/postgres.PostgresConfig\npevensie/postgres PostgresConfig","url":"pevensie/postgres.html#PostgresConfig"},{"doc":"pevensie/postgres","title":"PostgresError","content":"pub type PostgresError {\n  ConstraintViolated(\n    message: String,\n    constraint: String,\n    detail: String,\n  )\n  PostgresqlError(code: String, name: String, message: String)\n  ConnectionUnavailable\n}\n Errors that can occur when interacting with the Postgres driver.\n Will probably be removed or changed - haven&#39;t decided on the final API yet.\nConstraintViolated(\n  message: String,\n  constraint: String,\n  detail: String,\n)\n\n\nPostgresqlError(code: String, name: String, message: String)\n\n\nConnectionUnavailable\n\n\nSynonyms:\npevensie/postgres.PostgresError\npevensie/postgres PostgresError","url":"pevensie/postgres.html#PostgresError"},{"doc":"pevensie/postgres","title":"session_select_fields","content":"pub const session_select_fields: String\n The SQL used to select fields from the `session` table.\nSynonyms:\npevensie/postgres.session_select_fields\npevensie/postgres session_select_fields","url":"pevensie/postgres.html#session_select_fields"},{"doc":"pevensie/postgres","title":"user_select_fields","content":"pub const user_select_fields: String\n The SQL used to select fields from the `user` table.\nSynonyms:\npevensie/postgres.user_select_fields\npevensie/postgres user_select_fields","url":"pevensie/postgres.html#user_select_fields"},{"doc":"pevensie/postgres","title":"default_config","content":"pub fn default_config() -&gt; PostgresConfig\n Returns a default [`PostgresConfig`](#PostgresConfig) for connecting to a local\n Postgres database.\n\n Can also be used to provide sensible concurrency defaults for connecting\n to a remote database.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   // ...\n }\n \nSynonyms:\npevensie/postgres.default_config\npevensie/postgres default_config","url":"pevensie/postgres.html#default_config"},{"doc":"pevensie/postgres","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\npevensie/postgres.main\npevensie/postgres main","url":"pevensie/postgres.html#main"},{"doc":"pevensie/postgres","title":"new_auth_driver","content":"pub fn new_auth_driver(\n  config: PostgresConfig,\n) -&gt; AuthDriver(Postgres, PostgresError, a)\n Creates a new [`AuthDriver`](/pevensie/drivers/drivers.html#AuthDriver) for use with\n the [`pevensie/auth.new`](/pevensie/auth.html#new) function.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n import pevensie/auth.{type PevensieAuth}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_auth_driver(config)\n   let pevensie_auth = auth.new(\n     driver:,\n     user_metadata_decoder:,\n     user_metadata_encoder:,\n     cookie_key: &quot;super secret signing key&quot;,\n   )\n   // ...\n }\n \nSynonyms:\npevensie/postgres.new_auth_driver\npevensie/postgres new_auth_driver","url":"pevensie/postgres.html#new_auth_driver"},{"doc":"pevensie/postgres","title":"new_cache_driver","content":"pub fn new_cache_driver(\n  config: PostgresConfig,\n) -&gt; CacheDriver(Postgres, PostgresError)\n Creates a new [`CacheDriver`](/pevensie/drivers/drivers.html#CacheDriver) for use with\n the [`pevensie/cache.new`](/pevensie/cache.html#new) function.\n\n \n import pevensie/drivers/postgres.{type PostgresConfig}\n import pevensie/cache.{type PevensieCache}\n\n pub fn main() {\n   let config = PostgresConfig(\n     ..postgres.default_config(),\n     host: &quot;db.pevensie.dev&quot;,\n     database: &quot;my_database&quot;,\n   )\n   let driver = postgres.new_cache_driver(config)\n   let pevensie_cache = cache.new(driver)\n   // ...\n }\n \nSynonyms:\npevensie/postgres.new_cache_driver\npevensie/postgres new_cache_driver","url":"pevensie/postgres.html#new_cache_driver"},{"doc":"pevensie/postgres","title":"session_decoder","content":"pub fn session_decoder() -&gt; Decoder(Session)\n A decoder for the `session` table. Requires use of the\n [`session_select_fields`](#session_select_fields) when querying.\nSynonyms:\npevensie/postgres.session_decoder\npevensie/postgres session_decoder","url":"pevensie/postgres.html#session_decoder"},{"doc":"pevensie/postgres","title":"user_decoder","content":"pub fn user_decoder(\n  user_metadata_decoder: Decoder(a),\n) -&gt; Decoder(User(a))\n A decoder for the `user` table. Requires use of the\n [`user_select_fields`](#user_select_fields) when querying.\nSynonyms:\npevensie/postgres.user_decoder\npevensie/postgres user_decoder","url":"pevensie/postgres.html#user_decoder"},{"doc":"pevensie/postgres","title":"pevensie/postgres","content":"","url":"pevensie/postgres.html"}]);